@@ -111,9 +111,10 @@ class Emitter(object):
         block_seq_indent=None,
         top_level_colon_align=None,
         prefix_colon=None,
+        brace_single_entry_mapping_in_flow_sequence=None,
         dumper=None,
     ):
-        # type: (StreamType, Any, Optional[int], Optional[int], Optional[bool], Any, Optional[int], Optional[bool], Any, Any) -> None  # NOQA
+        # type: (StreamType, Any, Optional[int], Optional[int], Optional[bool], Any, Optional[int], Optional[bool], Any, Optional[bool], Any) -> None  # NOQA
         self.dumper = dumper
         if self.dumper is not None and getattr(self.dumper, '_emitter', None) is None:
             self.dumper._emitter = self
@@ -136,8 +137,9 @@ class Emitter(object):
         self.indents = Indents()
         self.indent = None  # type: Optional[int]
 
-        # Flow level.
-        self.flow_level = 0
+        # flow_context is an expanding/shrinking list consisting of '{' and '['
+        # for each unclosed flow context. If empty list that means block context
+        self.flow_context = []  # type: List[Text]
 
         # Contexts.
         self.root_context = False
@@ -154,14 +156,21 @@ class Emitter(object):
         self.column = 0
         self.whitespace = True
         self.indention = True
+        self.compact_seq_seq = True  # dash after dash
+        self.compact_seq_map = True  # key after dash
+        # self.compact_ms = False   # dash after key, only when excplicit key with ?
         self.no_newline = None  # type: Optional[bool]  # set if directly after `- `
 
-        # Whether the document requires an explicit document indicator
+        # Whether the document requires an explicit document end indicator
         self.open_ended = False
 
         # colon handling
         self.colon = u':'
         self.prefixed_colon = self.colon if prefix_colon is None else prefix_colon + self.colon
+        # single entry mappings in flow sequence
+        self.brace_single_entry_mapping_in_flow_sequence = (
+            brace_single_entry_mapping_in_flow_sequence
+        )  # NOQA
 
         # Formatting details.
         self.canonical = canonical
@@ -195,6 +204,8 @@ class Emitter(object):
         self.analysis = None  # type: Any
         self.style = None  # type: Any
 
+        self.scalar_after_indicator = True  # write a scalar on the same line as `---`
+
     @property
     def stream(self):
         # type: () -> Any
@@ -222,10 +233,15 @@ class Emitter(object):
         except AttributeError:
             return self  # cyaml
 
+    @property
+    def flow_level(self):
+        # type: () -> int
+        return len(self.flow_context)
+
     def dispose(self):
         # type: () -> None
         # Reset the state attributes (to clear self-references)
-        self.states = []  # type: List[Any]
+        self.states = []
         self.state = None
 
     def emit(self, event):
@@ -307,11 +323,11 @@ class Emitter(object):
             self.write_stream_start()
             self.state = self.expect_first_document_start
         else:
-            raise EmitterError('expected StreamStartEvent, but got %s' % self.event)
+            raise EmitterError('expected StreamStartEvent, but got %s' % (self.event,))
 
     def expect_nothing(self):
         # type: () -> None
-        raise EmitterError('expected nothing, but got %s' % self.event)
+        raise EmitterError('expected nothing, but got %s' % (self.event,))
 
     # Document handlers.
 
@@ -358,7 +374,7 @@ class Emitter(object):
             self.write_stream_end()
             self.state = self.expect_nothing
         else:
-            raise EmitterError('expected DocumentStartEvent, but got %s' % self.event)
+            raise EmitterError('expected DocumentStartEvent, but got %s' % (self.event,))
 
     def expect_document_end(self):
         # type: () -> None
@@ -370,7 +386,7 @@ class Emitter(object):
             self.flush_stream()
             self.state = self.expect_document_start
         else:
-            raise EmitterError('expected DocumentEndEvent, but got %s' % self.event)
+            raise EmitterError('expected DocumentEndEvent, but got %s' % (self.event,))
 
     def expect_document_root(self):
         # type: () -> None
@@ -388,19 +404,30 @@ class Emitter(object):
         if isinstance(self.event, AliasEvent):
             self.expect_alias()
         elif isinstance(self.event, (ScalarEvent, CollectionStartEvent)):
-            self.process_anchor(u'&')
+            if (
+                self.process_anchor(u'&')
+                and isinstance(self.event, ScalarEvent)
+                and self.sequence_context
+            ):
+                self.sequence_context = False
+            if root and isinstance(self.event, ScalarEvent) \
+               and not self.scalar_after_indicator:
+                self.write_indent()
             self.process_tag()
             if isinstance(self.event, ScalarEvent):
+                # nprint('@', self.indention, self.no_newline, self.column)
                 self.expect_scalar()
             elif isinstance(self.event, SequenceStartEvent):
+                # nprint('@', self.indention, self.no_newline, self.column)
+                i2, n2 = self.indention, self.no_newline  # NOQA
                 if self.event.comment:
                     if self.event.flow_style is False and self.event.comment:
                         if self.write_post_comment(self.event):
                             self.indention = False
                             self.no_newline = True
                     if self.write_pre_comment(self.event):
-                        self.indention = False
-                        self.no_newline = True
+                        self.indention = i2
+                        self.no_newline = not self.indention
                 if (
                     self.flow_level
                     or self.canonical
@@ -421,11 +448,11 @@ class Emitter(object):
                     or self.event.flow_style
                     or self.check_empty_mapping()
                 ):
-                    self.expect_flow_mapping()
+                    self.expect_flow_mapping(single=self.event.nr_items == 1)
                 else:
                     self.expect_block_mapping()
         else:
-            raise EmitterError('expected NodeEvent, but got %s' % self.event)
+            raise EmitterError('expected NodeEvent, but got %s' % (self.event,))
 
     def expect_alias(self):
         # type: () -> None
@@ -448,19 +475,20 @@ class Emitter(object):
         ind = self.indents.seq_flow_align(self.best_sequence_indent, self.column)
         self.write_indicator(u' ' * ind + u'[', True, whitespace=True)
         self.increase_indent(flow=True, sequence=True)
-        self.flow_level += 1
+        self.flow_context.append('[')
         self.state = self.expect_first_flow_sequence_item
 
     def expect_first_flow_sequence_item(self):
         # type: () -> None
         if isinstance(self.event, SequenceEndEvent):
             self.indent = self.indents.pop()
-            self.flow_level -= 1
+            popped = self.flow_context.pop()
+            assert popped == '['
             self.write_indicator(u']', False)
             if self.event.comment and self.event.comment[0]:
                 # eol comment on empty flow sequence
                 self.write_post_comment(self.event)
-            else:
+            elif self.flow_level == 0:
                 self.write_line_break()
             self.state = self.states.pop()
         else:
@@ -473,7 +501,8 @@ class Emitter(object):
         # type: () -> None
         if isinstance(self.event, SequenceEndEvent):
             self.indent = self.indents.pop()
-            self.flow_level -= 1
+            popped = self.flow_context.pop()
+            assert popped == '['
             if self.canonical:
                 self.write_indicator(u',', False)
                 self.write_indent()
@@ -493,11 +522,21 @@ class Emitter(object):
 
     # Flow mapping handlers.
 
-    def expect_flow_mapping(self):
-        # type: () -> None
+    def expect_flow_mapping(self, single=False):
+        # type: (Optional[bool]) -> None
         ind = self.indents.seq_flow_align(self.best_sequence_indent, self.column)
-        self.write_indicator(u' ' * ind + u'{', True, whitespace=True)
-        self.flow_level += 1
+        map_init = u'{'
+        if (
+            single
+            and self.flow_level
+            and self.flow_context[-1] == '['
+            and not self.canonical
+            and not self.brace_single_entry_mapping_in_flow_sequence
+        ):
+            # single map item with flow context, no curly braces necessary
+            map_init = u''
+        self.write_indicator(u' ' * ind + map_init, True, whitespace=True)
+        self.flow_context.append(map_init)
         self.increase_indent(flow=True, sequence=False)
         self.state = self.expect_first_flow_mapping_key
 
@@ -505,11 +544,14 @@ class Emitter(object):
         # type: () -> None
         if isinstance(self.event, MappingEndEvent):
             self.indent = self.indents.pop()
-            self.flow_level -= 1
+            popped = self.flow_context.pop()
+            assert popped == '{'  # empty flow mapping
             self.write_indicator(u'}', False)
             if self.event.comment and self.event.comment[0]:
                 # eol comment on empty mapping
                 self.write_post_comment(self.event)
+            elif self.flow_level == 0:
+                self.write_line_break()
             self.state = self.states.pop()
         else:
             if self.canonical or self.column > self.best_width:
@@ -528,14 +570,18 @@ class Emitter(object):
             # if self.event.comment and self.event.comment[1]:
             #     self.write_pre_comment(self.event)
             self.indent = self.indents.pop()
-            self.flow_level -= 1
+            popped = self.flow_context.pop()
+            assert popped in [u'{', u'']
             if self.canonical:
                 self.write_indicator(u',', False)
                 self.write_indent()
-            self.write_indicator(u'}', False)
+            if popped != u'':
+                self.write_indicator(u'}', False)
             if self.event.comment and self.event.comment[0]:
                 # eol comment on flow mapping, never reached on empty mappings
                 self.write_post_comment(self.event)
+            else:
+                self.no_newline = False
             self.state = self.states.pop()
         else:
             self.write_indicator(u',', False)
@@ -567,7 +613,12 @@ class Emitter(object):
 
     def expect_block_sequence(self):
         # type: () -> None
-        indentless = self.mapping_context and not self.indention
+        if self.mapping_context:
+            indentless = not self.indention
+        else:
+            indentless = False
+            if not self.compact_seq_seq and self.column != 0:
+                self.write_line_break()
         self.increase_indent(flow=False, sequence=True, indentless=indentless)
         self.state = self.expect_first_block_sequence_item
 
@@ -600,6 +651,8 @@ class Emitter(object):
 
     def expect_block_mapping(self):
         # type: () -> None
+        if not self.mapping_context and not (self.compact_seq_map or self.column == 0):
+            self.write_line_break()
         self.increase_indent(flow=False, sequence=False)
         self.state = self.expect_first_block_mapping_key
 
@@ -624,10 +677,15 @@ class Emitter(object):
                 if not isinstance(
                     self.event, (SequenceStartEvent, MappingStartEvent)
                 ):  # sequence keys
-                    if self.event.style == '?':
-                        self.write_indicator(u'?', True, indention=True)
+                    try:
+                        if self.event.style == '?':
+                            self.write_indicator(u'?', True, indention=True)
+                    except AttributeError:  # aliases have no style
+                        pass
                 self.states.append(self.expect_block_mapping_simple_value)
                 self.expect_node(mapping=True, simple_key=True)
+                if isinstance(self.event, AliasEvent):
+                    self.stream.write(u' ')
             else:
                 self.write_indicator(u'?', True, indention=True)
                 self.states.append(self.expect_block_mapping_value)
@@ -718,15 +776,18 @@ class Emitter(object):
     # Anchor, Tag, and Scalar processors.
 
     def process_anchor(self, indicator):
-        # type: (Any) -> None
+        # type: (Any) -> bool
         if self.event.anchor is None:
             self.prepared_anchor = None
-            return
+            return False
         if self.prepared_anchor is None:
             self.prepared_anchor = self.prepare_anchor(self.event.anchor)
         if self.prepared_anchor:
             self.write_indicator(indicator + self.prepared_anchor, True)
+            # issue 288
+            self.no_newline = False
         self.prepared_anchor = None
+        return True
 
     def process_tag(self):
         # type: () -> None
@@ -806,6 +867,7 @@ class Emitter(object):
         # if self.analysis.multiline and split    \
         #         and (not self.style or self.style in '\'\"'):
         #     self.write_indent()
+        # nprint('xx', self.sequence_context, self.flow_level)
         if self.sequence_context and not self.flow_level:
             self.write_indent()
         if self.style == '"':
@@ -890,13 +952,18 @@ class Emitter(object):
                 suffix = tag[len(prefix) :]
         chunks = []  # type: List[Any]
         start = end = 0
+        ch_set = u"-;/?:@&=+$,_.~*'()[]"
+        if self.dumper:
+            version = getattr(self.dumper, 'version', (1, 2))
+            if version is None or version >= (1, 2):
+                ch_set += u'#'
         while end < len(suffix):
             ch = suffix[end]
             if (
                 u'0' <= ch <= u'9'
                 or u'A' <= ch <= u'Z'
                 or u'a' <= ch <= u'z'
-                or ch in u"-;/?:@&=+$,_.~*'()[]"
+                or ch in ch_set
                 or (ch == u'!' and handle != u'!')
             ):
                 end += 1
@@ -985,6 +1052,8 @@ class Emitter(object):
                 if ch in u'?:':  # ToDo
                     if self.serializer.use_version == (1, 1):
                         flow_indicators = True
+                    elif len(scalar) == 1:  # single character
+                        flow_indicators = True
                     if followed_by_whitespace:
                         block_indicators = True
                 if ch == u'-' and followed_by_whitespace:
@@ -1339,21 +1408,42 @@ class Emitter(object):
 
     def determine_block_hints(self, text):
         # type: (Any) -> Any
-        hints = u""
+        indent = 0
+        indicator = u''
+        hints = u''
         if text:
-            if not self.root_context and text[0] in u' \n\x85\u2028\u2029':
-                hints += text_type(self.best_sequence_indent)
+            if text[0] in u' \n\x85\u2028\u2029':
+                indent = self.best_sequence_indent
+                hints += text_type(indent)
+            elif self.root_context:
+                for end in ['\n---', '\n...']:
+                    pos = 0
+                    while True:
+                        pos = text.find(end, pos)
+                        if pos == -1:
+                            break
+                        try:
+                            if text[pos + 4] in ' \r\n':
+                                break
+                        except IndexError:
+                            pass
+                        pos += 1
+                    if pos > -1:
+                        break
+                if pos > 0:
+                    indent = self.best_sequence_indent
             if text[-1] not in u'\n\x85\u2028\u2029':
-                hints += u'-'
+                indicator = u'-'
             elif len(text) == 1 or text[-2] in u'\n\x85\u2028\u2029':
-                hints += u'+'
-        return hints
+                indicator = u'+'
+        hints += indicator
+        return hints, indent, indicator
 
     def write_folded(self, text):
         # type: (Any) -> None
-        hints = self.determine_block_hints(text)
+        hints, _indent, _indicator = self.determine_block_hints(text)
         self.write_indicator(u'>' + hints, True)
-        if hints[-1:] == u'+':
+        if _indicator == u'+':
             self.open_ended = True
         self.write_line_break()
         leading_space = True
@@ -1365,7 +1455,7 @@ class Emitter(object):
             if end < len(text):
                 ch = text[end]
             if breaks:
-                if ch is None or ch not in u'\n\x85\u2028\u2029':
+                if ch is None or ch not in u'\n\x85\u2028\u2029\a':
                     if (
                         not leading_space
                         and ch is not None
@@ -1394,12 +1484,19 @@ class Emitter(object):
                         self.stream.write(data)
                     start = end
             else:
-                if ch is None or ch in u' \n\x85\u2028\u2029':
+                if ch is None or ch in u' \n\x85\u2028\u2029\a':
                     data = text[start:end]
                     self.column += len(data)
                     if bool(self.encoding):
                         data = data.encode(self.encoding)
                     self.stream.write(data)
+                    if ch == u'\a':
+                        if end < (len(text) - 1) and not text[end + 2].isspace():
+                            self.write_line_break()
+                            self.write_indent()
+                            end += 2  # \a and the space that is inserted on the fold
+                        else:
+                            raise EmitterError('unexcpected fold indicator \\a before space')
                     if ch is None:
                         self.write_line_break()
                     start = end
@@ -1410,7 +1507,7 @@ class Emitter(object):
 
     def write_literal(self, text, comment=None):
         # type: (Any, Any) -> None
-        hints = self.determine_block_hints(text)
+        hints, _indent, _indicator = self.determine_block_hints(text)
         self.write_indicator(u'|' + hints, True)
         try:
             comment = comment[1][0]
@@ -1418,7 +1515,7 @@ class Emitter(object):
                 self.stream.write(comment)
         except (TypeError, IndexError):
             pass
-        if hints[-1:] == u'+':
+        if _indicator == u'+':
             self.open_ended = True
         self.write_line_break()
         breaks = True
@@ -1434,8 +1531,12 @@ class Emitter(object):
                             self.write_line_break()
                         else:
                             self.write_line_break(br)
-                    if ch is not None and (not self.root_context or self.requested_indent):
-                        self.write_indent()
+                    if ch is not None:
+                        if self.root_context:
+                            idnx = self.indent if self.indent is not None else 0
+                            self.stream.write(u' ' * (_indent + idnx))
+                        else:
+                            self.write_indent()
                     start = end
             else:
                 if ch is None or ch in u'\n\x85\u2028\u2029':
@@ -1490,7 +1591,7 @@ class Emitter(object):
                         self.stream.write(data)
                     start = end
             elif breaks:
-                if ch not in u'\n\x85\u2028\u2029':
+                if ch not in u'\n\x85\u2028\u2029':  # type: ignore
                     if text[start] == u'\n':
                         self.write_line_break()
                     for br in text[start:end]:
@@ -1511,7 +1612,7 @@ class Emitter(object):
                     try:
                         self.stream.write(data)
                     except:  # NOQA
-                        print(repr(data))
+                        sys.stdout.write(repr(data) + '\n')
                         raise
                     start = end
             if ch is not None:
@@ -1519,11 +1620,11 @@ class Emitter(object):
                 breaks = ch in u'\n\x85\u2028\u2029'
             end += 1
 
-    def write_comment(self, comment):
-        # type: (Any) -> None
+    def write_comment(self, comment, pre=False):
+        # type: (Any, bool) -> None
         value = comment.value
-        # print('{:02d} {:02d} {!r}'.format(self.column, comment.start_mark.column, value))
-        if value[-1] == '\n':
+        # nprintf('{:02d} {:02d} {!r}'.format(self.column, comment.start_mark.column, value))
+        if not pre and value[-1] == '\n':
             value = value[:-1]
         try:
             # get original column position
@@ -1536,12 +1637,12 @@ class Emitter(object):
                 ValueError
         except ValueError:
             col = self.column + 1
-        # print('post_comment', self.line, self.column, value)
+        # nprint('post_comment', self.line, self.column, value)
         try:
             # at least one space if the current column >= the start column of the comment
             # but not at the start of a line
             nr_spaces = col - self.column
-            if self.column and value.strip() and nr_spaces < 1:
+            if self.column and value.strip() and nr_spaces < 1 and value[0] != '\n':
                 nr_spaces = 1
             value = ' ' * nr_spaces + value
             try:
@@ -1552,7 +1653,8 @@ class Emitter(object):
             self.stream.write(value)
         except TypeError:
             raise
-        self.write_line_break()
+        if not pre:
+            self.write_line_break()
 
     def write_pre_comment(self, event):
         # type: (Any) -> bool
@@ -1566,11 +1668,11 @@ class Emitter(object):
                     continue
                 if self.column != 0:
                     self.write_line_break()
-                self.write_comment(comment)
+                self.write_comment(comment, pre=True)
                 if isinstance(event, start_events):
                     comment.pre_done = True
         except TypeError:
-            print('eventtt', type(event), event)
+            sys.stdout.write('eventtt {} {}'.format(type(event), event))
             raise
         return True
 
