@@ -81,6 +81,7 @@ class Scanner(object):
             self.loader._scanner = self
         self.reset_scanner()
         self.first_time = False
+        self.yaml_version = None  # type: Any
 
     @property
     def flow_level(self):
@@ -155,10 +156,10 @@ class Scanner(object):
 
     @property
     def scanner_processing_version(self):  # prefix until un-composited
-        # type: () -> VersionType
+        # type: () -> Any
         if hasattr(self.loader, 'typ'):
-            return self.loader.resolver.processing_version  # type: ignore
-        return self.loader.processing_version  # type: ignore
+            return self.loader.resolver.processing_version
+        return self.loader.processing_version
 
     # Public methods.
 
@@ -453,7 +454,7 @@ class Scanner(object):
         # Reset simple keys.
         self.remove_possible_simple_key()
         self.allow_simple_key = False
-        self.possible_simple_keys = {}  # type: Dict[Any, Any]
+        self.possible_simple_keys = {}
         # Read the token.
         mark = self.reader.get_mark()
         # Add STREAM-END.
@@ -528,7 +529,7 @@ class Scanner(object):
         self.fetch_flow_collection_end(FlowMappingEndToken)
 
     def fetch_flow_collection_end(self, TokenClass):
-        # type: (Any, Text) -> None
+        # type: (Any) -> None
         # Reset possible simple key on the current level.
         self.remove_possible_simple_key()
         # Decrease the flow level.
@@ -797,7 +798,10 @@ class Scanner(object):
                 if self.flow_context[-1] == '[':
                     if self.reader.peek(1) not in _THE_END_SPACE_TAB:
                         return False
-                # if self.reader.peek(1) in '\'"{[]}':
+                elif self.tokens and isinstance(self.tokens[-1], ValueToken):
+                    # mapping flow context scanning a value token
+                    if self.reader.peek(1) not in _THE_END_SPACE_TAB:
+                        return False
                 return True
         # VALUE(block context): ':' (' '|'\n')
         return self.reader.peek(1) in _THE_END_SPACE_TAB
@@ -952,7 +956,8 @@ class Scanner(object):
                 "expected a digit or ' ', but found %r" % utf8(srp()),
                 self.reader.get_mark(),
             )
-        return (major, minor)
+        self.yaml_version = (major, minor)
+        return self.yaml_version
 
     def scan_yaml_directive_number(self, start_mark):
         # type: (Any) -> Any
@@ -1062,7 +1067,7 @@ class Scanner(object):
             ch = srp(length)
         if not length:
             raise ScannerError(
-                'while scanning an %s' % name,
+                'while scanning an %s' % (name,),
                 start_mark,
                 'expected alphabetic or numeric character, but found %r' % utf8(ch),
                 self.reader.get_mark(),
@@ -1074,7 +1079,7 @@ class Scanner(object):
         # assert ch1 == ch
         if ch not in '\0 \t\r\n\x85\u2028\u2029?:,[]{}%@`':
             raise ScannerError(
-                'while scanning an %s' % name,
+                'while scanning an %s' % (name,),
                 start_mark,
                 'expected alphabetic or numeric character, but found %r' % utf8(ch),
                 self.reader.get_mark(),
@@ -1132,8 +1137,8 @@ class Scanner(object):
         end_mark = self.reader.get_mark()
         return TagToken(value, start_mark, end_mark)
 
-    def scan_block_scalar(self, style):
-        # type: (Any) -> Any
+    def scan_block_scalar(self, style, rt=False):
+        # type: (Any, Optional[bool]) -> Any
         # See the specification for details.
         srp = self.reader.peek
         if style == '>':
@@ -1193,14 +1198,9 @@ class Scanner(object):
                 #
                 # This is the folding according to the specification:
 
-                if folded and line_break == '\n':
+                if rt and folded and line_break == '\n':
                     chunks.append('\a')
-                if (
-                    folded
-                    and line_break == '\n'
-                    and leading_non_space
-                    and srp() not in ' \t'
-                ):
+                if folded and line_break == '\n' and leading_non_space and srp() not in ' \t':
                     if not breaks:
                         chunks.append(' ')
                 else:
@@ -1239,7 +1239,7 @@ class Scanner(object):
             # Eat whitespaces and comments until we reach the next token.
             comment = self.scan_to_next_token()
             while comment:
-                trailing.append(comment[0])
+                trailing.append(' ' * comment[1].column + comment[0])
                 comment = self.scan_to_next_token()
 
             # Keep track of the trailing whitespace and following comments
@@ -1611,9 +1611,7 @@ class Scanner(object):
             line_break = self.scan_line_break()
             self.allow_simple_key = True
             prefix = self.reader.prefix(3)
-            if (prefix == '---' or prefix == '...') and srp(
-                3
-            ) in _THE_END_SPACE_TAB:
+            if (prefix == '---' or prefix == '...') and srp(3) in _THE_END_SPACE_TAB:
                 return
             breaks = []
             while srp() in ' \r\n\x85\u2028\u2029':
@@ -1622,9 +1620,7 @@ class Scanner(object):
                 else:
                     breaks.append(self.scan_line_break())
                     prefix = self.reader.prefix(3)
-                    if (prefix == '---' or prefix == '...') and srp(
-                        3
-                    ) in _THE_END_SPACE_TAB:
+                    if (prefix == '---' or prefix == '...') and srp(3) in _THE_END_SPACE_TAB:
                         return
             if line_break != '\n':
                 chunks.append(line_break)
@@ -1644,7 +1640,7 @@ class Scanner(object):
         ch = srp()
         if ch != '!':
             raise ScannerError(
-                'while scanning a %s' % name,
+                'while scanning a %s' % (name,),
                 start_mark,
                 "expected '!', but found %r" % utf8(ch),
                 self.reader.get_mark(),
@@ -1658,7 +1654,7 @@ class Scanner(object):
             if ch != '!':
                 self.reader.forward(length)
                 raise ScannerError(
-                    'while scanning a %s' % name,
+                    'while scanning a %s' % (name,),
                     start_mark,
                     "expected '!', but found %r" % utf8(ch),
                     self.reader.get_mark(),
@@ -1681,6 +1677,7 @@ class Scanner(object):
             or 'A' <= ch <= 'Z'
             or 'a' <= ch <= 'z'
             or ch in "-;/?:@&=+$,_.!~*'()[]%"
+            or ((self.scanner_processing_version > (1, 1)) and ch == "#")
         ):
             if ch == '%':
                 chunks.append(self.reader.prefix(length))
@@ -1696,7 +1693,7 @@ class Scanner(object):
             length = 0
         if not chunks:
             raise ScannerError(
-                'while parsing a %s' % name,
+                'while parsing a %s' % (name,),
                 start_mark,
                 'expected URI, but found %r' % utf8(ch),
                 self.reader.get_mark(),
@@ -1715,7 +1712,7 @@ class Scanner(object):
             for k in range(2):
                 if srp(k) not in '0123456789ABCDEFabcdef':
                     raise ScannerError(
-                        'while scanning a %s' % name,
+                        'while scanning a %s' % (name,),
                         start_mark,
                         'expected URI escape sequence of 2 hexdecimal numbers,'
                         ' but found %r' % utf8(srp(k)),
@@ -1732,7 +1729,7 @@ class Scanner(object):
             else:
                 value = unicode(b"".join(code_bytes), 'utf-8')
         except UnicodeDecodeError as exc:
-            raise ScannerError('while scanning a %s' % name, start_mark, str(exc), mark)
+            raise ScannerError('while scanning a %s' % (name,), start_mark, str(exc), mark)
         return value
 
     def scan_line_break(self):
@@ -1800,7 +1797,7 @@ class RoundTripScanner(Scanner):
             if isinstance(self.tokens[0], CommentToken):
                 self.tokens_taken += 1
                 comment = self.tokens.pop(0)
-                # print 'dropping2', comment
+                # nprint('dropping2', comment)
                 comments.append(comment)
         if len(comments) >= 1:
             self.tokens[0].add_pre_comments(comments)
@@ -1830,7 +1827,34 @@ class RoundTripScanner(Scanner):
                 and self.tokens[0].end_mark.line == self.tokens[1].start_mark.line
             ):
                 self.tokens_taken += 1
-                self.tokens[0].add_post_comment(self.tokens.pop(1))
+                c = self.tokens.pop(1)
+                self.fetch_more_tokens()
+                while len(self.tokens) > 1 and isinstance(self.tokens[1], CommentToken):
+                    self.tokens_taken += 1
+                    c1 = self.tokens.pop(1)
+                    c.value = c.value + (' ' * c1.start_mark.column) + c1.value
+                    self.fetch_more_tokens()
+                self.tokens[0].add_post_comment(c)
+            elif (
+                len(self.tokens) > 1
+                and isinstance(self.tokens[0], ScalarToken)
+                and isinstance(self.tokens[1], CommentToken)
+                and self.tokens[0].end_mark.line != self.tokens[1].start_mark.line
+            ):
+                self.tokens_taken += 1
+                c = self.tokens.pop(1)
+                c.value = (
+                    '\n' * (c.start_mark.line - self.tokens[0].end_mark.line)
+                    + (' ' * c.start_mark.column)
+                    + c.value
+                )
+                self.tokens[0].add_post_comment(c)
+                self.fetch_more_tokens()
+                while len(self.tokens) > 1 and isinstance(self.tokens[1], CommentToken):
+                    self.tokens_taken += 1
+                    c1 = self.tokens.pop(1)
+                    c.value = c.value + (' ' * c1.start_mark.column) + c1.value
+                    self.fetch_more_tokens()
             self.tokens_taken += 1
             return self.tokens.pop(0)
         return None
@@ -1883,6 +1907,10 @@ class RoundTripScanner(Scanner):
                 while ch not in _THE_END:
                     ch = srp()
                     if ch == '\0':  # don't gobble the end-of-stream character
+                        # but add an explicit newline as "YAML processors should terminate
+                        # the stream with an explicit line break
+                        # https://yaml.org/spec/1.2/spec.html#id2780069
+                        comment += '\n'
                         break
                     comment += ch
                     srf()
@@ -1940,6 +1968,10 @@ class RoundTripScanner(Scanner):
             return ch
         return ""
 
+    def scan_block_scalar(self, style, rt=True):
+        # type: (Any, Optional[bool]) -> Any
+        return Scanner.scan_block_scalar(self, style, rt=rt)
+
 
 # try:
 #     import psyco
